C
C     This program searches the Hebrew dictionary for a given
C     =transliteration= or "English translation" and reports
C     the closest matches.
C
C     unit  purpose
C     ----  -------
C       0   error messages
C       1   ${HOME}/Utility/hebrew.hsh
C       6   results output to the screen
C
C     variable  meaning
C     --------  -------
C     ARG       the command-line argument
C     DMAX1     Fortran function returns larger of REAL*8s
C     ENG       an English translation
C     ENGS      the English translations
C     GETARG    system routine gets one command-line argument
C     GETENG    routine tables English translations of Hebrew words
C     I         index on the words
C     IARGC     system function for number of command-line arguments
C     IFIX      Fortran function returns INTEGER*4 for REAL*4
C     ISTART    index of worst match
C     K         index on the characters of a string
C     LE        length of an English translation
C     LENGTH    function returns index of last nonblank in a string
C     LH        length of a Hebrew transliteration
C     LM        length of a possible meaning
C     LORD      sorted order of the transliterations matching string
C     LTG       length of target string
C     LTR       length of longest transliteration in report
C     MNG       a possible English meaning
C     MXWD      maximum numbrer of words allowed
C     NARG      number of command-line arguments found
C     NMATCH    number of word matches reported
C     NMAX      maximum number of word matches to report
C     NMIN      minimum number of word matches to report
C     NWD       number of words found in dictionary
C     PHRSIM    function scores phrases for similarity
C     RC        return code; 0 => all went well
C     SCORE     % match of transliteration or translation to TARGET
C     SHIFTL    routine removes leading blanks from a string
C     SIM       similarity score from PHRSIM
C     SIMAX     highest similarity score among possible meanings
C     SMIN      lowest score printed
C     SNGL      Fortran function returns REAL*4 for REAL*8
C     SORTI4    routine sorts an INTEGER*4 vector in ascending order
C     STRCPY    routine copies one string into another
C     STRSIM    function scores strings for similarity
C     TARGET    transliteration to be found
C     TRANS     the Hebrew transliterations
C     UNPUNC    routine removes punctuation & parenthesized strings
C
C     prepare to store translations and transliterations
      PARAMETER(MXWD=6000)
      CHARACTER*1 ENGS(80,MXWD)/480000*' '/,ENG(80),MNG(80)
      CHARACTER*1 TRANS(18,MXWD)/108000*' '/
C
C     prepare to get the string to be searched for
      CHARACTER*80 ARG
      CHARACTER*1 TARGET(80)
      EQUIVALENCE(ARG,TARGET)
C
C     prepare to compute and store the scores
      INTEGER*4 RC,SCORE(MXWD),LORD(MXWD),SMIN
      REAL*8 STRSIM,SIM,SIMAX,PHRSIM
C
C ------------------------------------------------------------------
C
C     get transliterations and translations from the dictionary
      CALL GETENG(MXWD, NWD,TRANS,ENGS)
C
C     get the target phrase to be searched for
      NARG=IARGC()
      IF(NARG.NE.1) THEN
         WRITE(0,901)
  901    FORMAT('usage: hebcheck =transliteration='/
     ;          '       hebcheck "translation"'/
     ;          '       finds the closest matches')
         STOP
      ENDIF
      CALL GETARG(1,ARG)
      LTG=LENGTH(TARGET,80)
      IF((TARGET(1).EQ.'=' .AND. TARGET(LTG).NE.'=')  .OR.
     ;   (TARGET(1).NE.'=' .AND. TARGET(LTG).EQ.'=')) THEN
          WRITE(0,901)
          STOP
      ENDIF
C
C     score each dictionary entry for its match to the target
      IF(TARGET(1).EQ.'=' .AND. TARGET(LTG).EQ.'=') THEN
         TARGET(1)=' '
         TARGET(LTG)=' '
         CALL SHIFTL(TARGET,LTG)
         LTG=LTG-2
C        compare the target word to each Hebrew transliteration
         DO 1 I=1,NWD
              LH=LENGTH(TRANS(1,I),18)
              SCORE(I)=IFIX(0.5+SNGL(100.D0*
     ;                 STRSIM(TARGET,LTG,TRANS(1,I),LH)))
    1    CONTINUE
      ELSE
C        compare the target phrase to each English translation
         DO 2 I=1,NWD
C             prepare to parse out the translation
              CALL STRCPY(ENGS(1,I),80,80,0, ENG)
              SIMAX=0.D0
              LE=LENGTH(ENG,80)
              IF(LE.EQ.0) GO TO 3
C
C             consider each alternate meaning in the translation
    4         K=0
    5         K=K+1
              MNG(K)=ENG(K)
              IF(K.EQ.LE .OR. ENG(K).EQ.',' .OR. ENG(K).EQ.';') THEN
                 CALL UNPUNC(MNG,K)
                 LM=LENGTH(MNG,K)
                 SIM=PHRSIM(TARGET,LTG,MNG,LM)
                 SIMAX=DMAX1(SIM,SIMAX)
                 ENG(K)=' '
                 CALL SPACEL(ENG,LE)
                 LE=LENGTH(ENG,LE)
                 IF(LE.EQ.0) GO TO 3
                 GO TO 4
              ENDIF
              ENG(K)=' '
              GO TO 5
C
C             save the highest match score
    3         SCORE(I)=IFIX(0.5+SNGL(100.D0*SIMAX))
    2    CONTINUE
      ENDIF
C
C     sort the scores
      CALL SORTI4(NWD, SCORE, LORD)
C
C     figure out how to format the report
      SMIN=10
      NMIN=3
      NMAX=30
      NMATCH=0
      LTR=0
      DO 6 I=NWD,1,-1
           ISTART=I
           IF(SCORE(I).EQ.0) GO TO 7
           IF(SCORE(I).LT.SMIN .AND. NMATCH.GE.NMIN) GO TO 7
           IF(NMATCH.GE.NMAX) GO TO 7
           LTR=MAX0(LTR,LENGTH(TRANS(1,LORD(I)),18))
           NMATCH=NMATCH+1
    6 CONTINUE
C
C     report the highest scores
    7 DO 8 I=NWD,ISTART,-1
           LE=LENGTH(ENGS(1,LORD(I)),80)
           WRITE(6,902) LORD(I),SCORE(I),
     ;                  (TRANS(K,LORD(I)),K=1,LTR),
     ;                  ' ',':',' ',
     ;                  (ENGS(K,LORD(I)),K=1,LE)
  902      FORMAT(1X,I4.4,2X,I3,'%',2X,121A1)
    8 CONTINUE
      STOP
      END
