C
C     This routine reads from standard in transliterated Hebrew
C     written in one direction and writes on standard out the
C     corresponding transliterated Hebrew written in the other
C     direction.
C
C     variable  meaning
C     --------  -------
C     HTPWID    routine separates subtokens in a Hebrew text string
C     I         index on the substrings in a token
C     IN        input text line
C     J         index in OUT where a subtoken begins
C     JSTART    index in OUT where a token begins
C     K         index on the characters of a string
C     L         length of an input line
C     LENGTH    function returns index of last nonblank in a string
C     LNUM      input file line number just read
C     LS        length of a subtoken
C     LTK       length of a token
C     NST       number of subtokens found in the current token
C     OUT       output text line
C     RC        return codes; 0 => all went well
C     SHFTKN    routine shifts string left by given number of tokens
C     SHIFTL    routine removes leading blanks from a string
C     SUBS      subtokens found in the current token
C     TOKEN     token composed of Hebrew, <transliteration>, punct
C     WTK       unused; printed width of the current token
C
      CHARACTER*1 IN(1024),OUT(1024),TOKEN(1024),SUBS(20,10)
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     get the next input line
      LNUM=0
    7 READ(5,901,END=1) IN
  901 FORMAT(1024A1)
      LNUM=LNUM+1
      L=LENGTH(IN,1024)
C
C     assembly of the corresponding output line starts at the right
      JSTART=1024
      DO 8 K=JSTART,1,-1
           OUT(K)=' '
    8 CONTINUE
C
C     copy out the input token that is next reading L to R
    6 LTK=0
      DO 2 K=1,L 
           IF(IN(K).EQ.' ') GO TO 3
           LTK=LTK+1
           TOKEN(K)=IN(K)
    2 CONTINUE
C
C     find its subtokens
    3 CALL HTPWID(TOKEN,LTK,12.0, SUBS,NST,WTK,RC)
      IF(RC.NE.0) THEN
         WRITE(0,902) LNUM,RC,(TOKEN(K),K=1,LTK)
  902    FORMAT('on input line ',I3,' HTPWID returned RC=',I3,
     ;          ' for token ',1024A1)
         STOP
      ENDIF
C
C     assemble the subtokens in reverse order, R to L in output
      DO 4 I=1,NST
           LS=LENGTH(SUBS(1,I),20)
           J=JSTART-LS
           DO 5 K=1,LS
                OUT(J+K)=SUBS(K,I)
    5      CONTINUE
           JSTART=J
    4 CONTINUE
C
C     shift this token out of the input line and go for the next
      CALL SHFTKN(IN,L,1,RC)
      L=LENGTH(IN,L)
      IF(L.GT.0) THEN
        OUT(JSTART)=' '
        JSTART=JSTART-1
        GO TO 6
      ENDIF
C
C     the end of the input line has been reached
      CALL SHIFTL(OUT,1024)
      L=LENGTH(OUT,1024)
      WRITE(6,901) (OUT(K),K=1,L)
      GO TO 7
C
C     there are no more input lines
    1 STOP
      END
