C
C     This program reads a .raw LaTeX file containing Hebrew text
C     including <transliterations> and produces a .heb LaTeX file
C     that prints text left-hand pages and lexicon right-hand pages.
C
C     unit  assignment
C     ----  ----------
C       0   errors
C       1   input LaTeX file, which this program rewinds
C       2   ${HOME}/Utility/hebrew.hsh momentarily
C       3   transliterations in (from previous runs)
C       4   transliterations out (including new this run)
C       6   output LaTeX file including lexicon opposite text
C
C     variable  meaning
C     --------  -------
C     ALL       list of all transliterations in the document
C     BF        T => print lexicon line in bold
C     DONE      T => the end of the input file has been reached
C     FONT      font size specified in input file, or 12
C     GETFIL    routine attaches a file for a given purpose
C     GETIN     routine reads the next input line
C     I         index on transliterations
C     IEND      number of final input line in this output page
C     ISTART    number of first input line in this output page
C     J         second index on transliterations
C     MXALL     maximum transliterations allowed in document
C     MXPG      transliterations that will fit in a lexicon page
C     MXTL      maximum transliterations allowed in a line
C     MXTN      maximum new transliterations allowed
C     NALL      # transliterations in document
C     NEW       routine lists transliterations new to current page
C     NPG       # transliterations in current lexicon page
C     NTL       # transliterations in in current input line
C     NTN       # trans in current input line new to this page
C     PUTOL     routine writes a lexicon page
C     PUTOX     routine writes a text page
C     TLINE     transliterations found in the current input line
C     TNEW      new transliterations unique to current output page
C     TPAGE     transliterations occurring on current output page
C     TRN       a transliteration
C
C     store lists of transliterations
      PARAMETER(MXPG=42,MXTL=MXPG,MXTN=MXPG)
      CHARACTER*18 TLINE(MXTL),TPAGE(MXPG),TNEW(MXTN)
      PARAMETER(MXALL=1000)
      CHARACTER*18 ALL(MXALL)
C
C     other variables
      LOGICAL*4 BF(MXPG),DONE
      CHARACTER*1 FONT(2)/'1','2'/
      CHARACTER*18 TRN
C
C ------------------------------------------------------------------
C
C     read the list of all transliterations in this document so far
      CALL GETFIL('transin',7,' ',0,3,1)
      NALL=0
    2 READ(3,902,END=1) TRN
  902 FORMAT(A18)
      NALL=NALL+1
      ALL(NALL)=TRN
      GO TO 2
C
C     attach the input .raw LaTeX file for this chapter
    1 CALL GETFIL('input',5,' ',0,1,1)
C
C     initialize
      IEND=0
      ISTART=IEND+1
      NPG=0
C
C     read a line of input and list its unique transliterations
    3 CALL GETIN(TLINE,MXTL,NTL,IEND,FONT,DONE)
        IF(DONE) GO TO 11
C
C       figure out which of them would be new to the current page
        CALL NEW(TLINE,MXTL,NTL,TPAGE,MXPG,NPG, TNEW,MXTN,NTN)
C
C       will the new lines fit in the lexicon?
        IF(NPG+NTN .LE. MXPG) GO TO 21
        IEND=IEND-1
C
C       lexicon is full
C         mark transliterations new to the document to print bold
   11     DO 12 I=1,NPG
                DO 13 J=1,NALL
                      IF(TPAGE(I).EQ.ALL(J)) THEN
                         BF(I)=.FALSE.
                         GO TO 12
                      ENDIF
   13            CONTINUE
                 BF(I)=.TRUE.
C
C                and update the list of transliterations in document
                 NALL=NALL+1
                 ALL(NALL)=TPAGE(I)
   12     CONTINUE
C
C         write the text page
          CALL PUTOX(FONT,ISTART,IEND)
C
C         write the corresponding lexicon page
          CALL PUTOL(TPAGE,MXPG,NPG,BF,FONT)
          IF(DONE) GO TO 4
C
C         start a new output page with the current input line
          NPG=0
          DO 14 I=1,NTL
                NPG=NPG+1
                TPAGE(NPG)=TLINE(I)
   14     CONTINUE 
          GO TO 3
C
C       lexicon has room for the new transliterations
C         add them to the current lexicon page
   21     DO 22 I=1,NTN
                NPG=NPG+1
                TPAGE(NPG)=TNEW(I)
   22     CONTINUE
          GO TO 3
C
C     update the file of all transliterations in the document
    4 CALL GETFIL('transout',8,' ',0,4,2)
      DO 5 J=1,NALL
           WRITE(4,902) ALL(J)
    5 CONTINUE
      STOP
      END
C
      SUBROUTINE GETIN(TLINE,MXTL,NTL,IEND,FONT,DONE)
C     This routine finds the next input line that contains a
C     transliteration and returns in TLINE all of the
C     different transliteraions that it contains.
C
C     variable  meaning
C     --------  -------
C     BLANKS    the string '                  '
C     DONE      T => the end of the input file has been reached
C     FINDST    routine finds starting location of a substring
C     FONT      Hebrew font size specified in input .raw LaTeX file
C     I         index on transliterations
C     IEND      line number of last input line read
C     J         second index on transliterations
C     JJ        third index on transliterations for copying
C     KSTART    position in LINE where \setivrit starts, or 0
C     L         length of LINE
C     LENGTH    function returns index of last nonblank in a string
C     LINE      text line read from input .raw LaTeX file
C     LISTRN    routine lists all transliterations found in a string
C     MXTL      dimensioned size of TLINE
C     NTL       number of transliterations found in LINE
C     TLINE     the transliterations found in LINE
C
C     formal parameters
      CHARACTER*18 TLINE(MXTL)
      CHARACTER*1 FONT(2)
      LOGICAL*4 DONE
C
C     local variables
      CHARACTER*1 LINE(1024)
      CHARACTER*18 BLANKS/'                  '/
C
C ------------------------------------------------------------------
C
C     read the next input line
    3 READ(1,901,END=1) LINE
  901 FORMAT(1024A1)
      IEND=IEND+1
      GO TO 2
    1 DONE=.TRUE.
      RETURN
    2 L=LENGTH(LINE,1024)
C
C     skip blank lines and comments
      IF(L.EQ.0) GO TO 3
      IF(LINE(1).EQ.'%') GO TO 3
C
C     list all transliterations that are in this input line
      CALL LISTRN(LINE,L,MXTL, TLINE,NTL)
C
C     stop if an error occurred in parsing the input line
      IF(NTL.LT.0) THEN
         WRITE(0,902) IEND
  902    FORMAT('bad transliteration syntax on line',I4)
         STOP
      ENDIF
C
C     look for a font specification if no transliterations
      IF(NTL.EQ.0) THEN
         CALL FINDST(LINE,L,'\setivrit{',10, KSTART)
         IF(KSTART.NE.1) GO TO 3
C
C        yes; decode it
         IF(LINE(11).EQ.'}') THEN
C           only one digit in the font size
            FONT(1)=' '
            FONT(2)=LINE(11)
         ELSE
C           two digits in the font size
            FONT(1)=LINE(11)
            FONT(2)=LINE(12)
         ENDIF
         GO TO 3
      ENDIF
C
C     remove duplicates from the list of transliterations
      NTL=NTL+1
    7 NTL=NTL-1
      DO 4 I=1,NTL-1
           DO 5 J=I+1,NTL
                IF(TLINE(J).EQ.TLINE(I)) THEN
C                  copy rows up to eliminate row J
                   DO 6 JJ=J,NTL-1
                        TLINE(JJ)=TLINE(JJ+1)
    6              CONTINUE
C                  and blank out the now-duplicate bottom line
                   TLINE(NTL)=BLANKS
                   GO TO 7
                ENDIF
    5      CONTINUE
    4 CONTINUE
      RETURN
      END
C
      SUBROUTINE NEW(TLINE,MXTL,NTL,TPAGE,MXPG,NPG, TNEW,MXTN,NTN)
C     This routine lists the translations in the current input line
C     that are new to this lexicon page.
C
C     variable  meaning
C     --------  -------
C     I         index on transliterations in current input line
C     J         index on transliterations in current page so far
C     MXPG      dimensioned size of TPAGE
C     MXTL      dimensioned size of TLINE
C     MXTN      dimensioned size of TNEW
C     NPG       # transliterations in current output page
C     NTL       # transliterations in current input line
C     NTN       # input line transliterations new to output page
C     TLINE     transliterations in current input line
C     TNEW      input line transliterations new to output page
C     TPAGE     transliterations in current output page
C
C     formal parameters
      CHARACTER*18 TLINE(MXTL),TPAGE(MXPG),TNEW(MXTN)
C
C ------------------------------------------------------------------
C
C     no new transliterations have been found yet
      NTN=0
C
C     consider each transliteration in the current input line
      DO 1 I=1,NTL
C          is this transliteration in the current output page?
           DO 2 J=1,NPG
                IF(TLINE(I).EQ.TPAGE(J)) GO TO 1
    2      CONTINUE
C
C          no; list it as new to this lexicon
           NTN=NTN+1
           TNEW(NTN)=TLINE(I)
    1 CONTINUE
      RETURN
      END
C
      SUBROUTINE PUTOL(TPAGEC,MXPG,NPG,BF,FONT)
C     This routine writes a lexicon page.
C
C     variable  meaning
C     --------  -------
C     BF        T => print this line in boldface
C     ENGTRN    finds translations corresponding to transliterations
C     EPAGEC    translations for this lexicon page
C     FONT      font size specified in input .raw LaTeX file
C     I         index on transliterations and translations
C     K         index on the characters of a string
C     LE        length of a translation
C     LENGTH    function returns index of last nonblank in a string
C     LT        length of a transliteration
C     MXPG      maximum transliterations in a page
C     NPG       number of transliterations in this page
C     RC        return code; 0 => all went well
C     TPAGEC    transliterations in this page
C
C     formal parameters
      CHARACTER*1 TPAGEC(18,MXPG)
      LOGICAL*4 BF(MXPG)
      CHARACTER*1 FONT(2)
C
C     automatic local variable
      CHARACTER*1 EPAGEC(80,MXPG)
C
C     other local variables
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     look up the translations corresponding to the transliterations
      CALL ENGTRN(TPAGEC,NPG,2, EPAGEC,RC)
      IF(RC.GT.0) THEN
         LT=LENGTH(TPAGEC(1,RC),18)
         WRITE(0,901) '<',(TPAGEC(K,RC),K=1,LT),'>'
  901    FORMAT('transliteration not in dictionary: ',20A1)
         STOP
      ENDIF
C
C     start the LaTeX for this page
      WRITE(6,906)
  906 FORMAT('\addtocounter{page}{-1}')
C
C     generate LaTeX for the lexicon page
      WRITE(6,902)
  902 FORMAT('\setivrit{12}'/
     ;       '\begin{figure}[H]'/
     ;       '\centering'/
     ;       '\begin{tabular}{rl}')
      DO 1 I=1,NPG
           LT=LENGTH(TPAGEC(1,I),18)
           LE=LENGTH(EPAGEC(1,I),80)
           IF(BF(I)) THEN
              WRITE(6,903) '\renewcommand{\hbold}[1]{\pmb{#1}} <',
     ;        (TPAGEC(K,I),K=1,LT),
     ;        '>',' ','&',' ','{','\','b','f',' ',
     ;        (EPAGEC(K,I),K=1,LE),
     ;        '}','\','\'
  903         FORMAT(A36,110A1)
           ELSE
              WRITE(6,904) '<',
     ;        (TPAGEC(K,I),K=1,LT),
     ;        '>',' ','&',' ','{',
     ;        (EPAGEC(K,I),K=1,LE),
     ;        '}','\','\'
  904         FORMAT(108A1)
           ENDIF
           BF(I)=.FALSE.
    1 CONTINUE
      WRITE(6,905) FONT
  905 FORMAT('\end{tabular}'/
     ;       '\end{figure}'/
     ;       '\setivrit{',2A1,'}'/
     ;       '\newpage')
      RETURN
      END
C
      SUBROUTINE PUTOX(FONT,ISTART,IEND)
C     This routine writes a text page.
C
C     variable  meaning
C     --------  -------
C     DTB       routine converts numerals to an INTEGER*4
C     FINDST    routine finds starting location of a substring
C     FLOAT     Fortran function returns REAL*4 for INTEGER*4
C     FONT      Hebrew font size as numerals
C     I         index on input lines
C     IEND      final input line corresponding to this page pair
C     ISTART    first input line corresponding to this page pair
C     K         index on the characters of LINE
C     KSTART    position in LINE where \hebline is found, or 0
C     L         length of LINE
C     LENGTH    function returns index of last nonblank in a string
C     LFONT     Hebrew font size as an integer
C     LINE      an input line of Hebrew text
C     LSPACE    real space between lines of text in points
C     RC        return code from DTB; 0 => all went well
C
      CHARACTER*1 LINE(1024),FONT(2)
      REAL*4 LSPACE
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     reposition to the first input line for this output page
      REWIND(1)
      DO 1 I=1,ISTART-1
           READ(1,*)
    1 CONTINUE
C
C     figure out the linespacing from the font size
      CALL DTB(FONT,2, LFONT,RC)
      LSPACE=0.7*FLOAT(LFONT)
C
C     for each input line
      DO 2 I=ISTART,IEND
           READ(1,902) LINE
  902      FORMAT(1024A1)
C
C          copy it to the output
           L=LENGTH(LINE,1024)
           WRITE(6,902) (LINE(K),K=1,L)
C
C          does this line begin with \hebline{?
           CALL FINDST(LINE,L,'\hebline{',9, KSTART)
           IF(KSTART.NE.1) GO TO 2
C
C          yes; double space the lines
           IF(I.LT.IEND) THEN
              WRITE(6,903) LSPACE
  903         FORMAT(' '/'\vspace{',F4.1,'pt}')
           ENDIF
    2 CONTINUE
C
C     end the LaTeX for this page
      WRITE(6,904)
  904 FORMAT('\newpage')
C
C     advance the starting input line number
      ISTART=IEND+1
      RETURN
      END
