C
C     This program expands the Hebrew hash codes on a given line of
C     an input .hsh dictionary file, and writes on standard out the
C     the transliteration of the word, the LaTeX to typeset it, and
C     its English translation.
C
C     unit  purpose
C     ----  -------
C       0   error messages
C       5   hashed dictionary file
C       6   extracted LaTeX
C
C     variable  meaning
C     --------  -------
C     ARG       command-line argument
C     DTB       routine converts numerals to an INTEGER*4
C     ENG       English translation of the word
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     HSH2HB    routine expands the hash of a Hebrew string
C     HSTR      Hebrew string
C     I         index on the lines of the hash file
C     K         index on the characters of a string
C     L         length of English translation
C     LENGTH    function returns index of last nonblank in a string
C     LHSTR     length of Hebrew string expanded from hash
C     LNR       line number of the word that is wanted
C     LTR       hash codes for the letters in the word
C     MIN0      Fortran function returns smaller of INTEGER*4s
C     MXCH      number of letters or vowels allowed
C     MXHSTR    maximum number of characters allowed in result
C     RC        return codes; 0 => all went well
C     SHIFTR    routine makes string flush right in its length
C     TEXT      the hash file line as text
C     TRN       the transliteration
C     VWL       hash codes for the vowels in the word
C
C     prepare to reread hash file lines
      CHARACTER*256 TEXT
C
C     dimension hash code and result strings
      PARAMETER(MXCH=12,MXHSTR=256)
      INTEGER*1 LTR(MXCH),VWL(MXCH),HSTR(MXHSTR)
      CHARACTER*1 TRN(18)/18*' '/,ENG(80)/80*' '/
C
C     other variables
      CHARACTER*4 ARG
      INTEGER*4 RC
C
C ------------------------------------------------------------------
C
C     get the number of the line to expand
      CALL GETARG(1,ARG)
      CALL SHIFTR(ARG,4)
      CALL DTB(ARG,4, LNR,RC)
      IF(RC.EQ.0 .AND. LNR.GT.0) GO TO 1
      WRITE(0,991) ARG
  991 FORMAT('bogus line number ',A4)
      CALL EXIT(1)
C
C     skip down to that line and read it
    1 DO 2 I=1,LNR-1
           READ(5,*,END=3)
    2 CONTINUE
      READ(5,901,END=3) TEXT
  901 FORMAT(A256)
      L=MIN0(80,LENGTH(TEXT,256)-64)
      READ(TEXT,902) TRN,(VWL(K),LTR(K),K=1,MXCH),(ENG(K),K=1,L)
  902 FORMAT(18A1,4X,12(2Z2),80A1)
      GO TO 4
    3 WRITE(0,992) I
  992 FORMAT('past end of hash file at I=',I4)
      CALL EXIT(2)
C
C     expand the hash codes into LaTeX
    4 CALL HSH2HB(MXCH,LTR,VWL,MXHSTR, LHSTR,HSTR,RC)
      IF(RC.EQ.0) GO TO 5
      WRITE(0,993) VWL(RC),LTR(RC),VWL(RC),LTR(RC),RC
  993 FORMAT('bad hash code Z',2Z2.2,'=',2I2.2,' at character ',I2)
      WRITE(0,901) TEXT
      CALL EXIT(3)
C
C     output the result
    5 WRITE(6,903) TRN,' ',(HSTR(K),K=1,LHSTR),' ',(ENG(K),K=1,L)
  903 FORMAT(256A1)
      CALL EXIT(0)
      END
